<?php

namespace Drupal\purge_purger_keycdn\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\purge_ui\Form\PurgerConfigFormBase;
use GuzzleHttp\ClientInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configuration form for the Key CDN Purger.
 */
class KeyCDNPurgerConfigForm extends PurgerConfigFormBase {

  /**
   * The http client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * Constructs an instance of KeyCDNPurgerConfigForm.
   *
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The http client.
   */
  public function __construct(ClientInterface $http_client) {
    $this->httpClient = $http_client;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('http_client')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['purge_purger_keycdn.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'purge_purger_keycdn.config_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Key'),
      '#required' => TRUE,
      '#description' => $this->t('Enter the api key here.'),
      '#default_value' => $this->config('purge_purger_keycdn.settings')->get('api_key'),
    ];
    $form['zone'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Zone'),
      '#required' => TRUE,
      '#description' => $this->t('Enter the zone id here.'),
      '#default_value' => $this->config('purge_purger_keycdn.settings')->get('zone'),
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $api_key = $form_state->getValue('api_key');
    $zone_id = $form_state->getValue('zone');
    $uri = 'https://api.keycdn.com/zones.json';

    // If api key and zone is set.
    if (!empty($api_key) && !empty($zone_id)) {
      try {
        $response = $this->httpClient->request('GET', $uri, [
          'auth' => [$api_key, ''],
        ]);

        // If key is not valid.
        if ($response->getStatusCode() != 200) {
          $form_state->setErrorByName('api_key', $this->t('Please check api key. It is either invalid or wrong.'));
        }

        // Validate the zone.
        $output = json_decode($response->getBody());
        if (empty($output->data->zones)) {
          $form_state->setErrorByName('zone', $this->t('Invalid zone id.'));
        }
        else {
          $valid_zone = FALSE;
          foreach ($output->data->zones as $zone) {
            // If zone id matches, no need to process the loop further.
            if ($zone->id == $zone_id) {
              $valid_zone = TRUE;
              break;
            }
          }

          // If not a valid zone.
          if (!$valid_zone) {
            $form_state->setErrorByName('zone', $this->t('Invalid zone id.'));
          }
        }
      }
      catch (\Exception $e) {
        $form_state->setErrorByName('api_key', $this->t('It looks like Key CDN is not responding or something is wrong.'));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitFormSuccess(array &$form, FormStateInterface $form_state) {
    $this->config('purge_purger_keycdn.settings')
      ->set('api_key', $form_state->getValue('api_key'))
      ->set('zone', $form_state->getValue('zone'))
      ->save();
  }

}
